{ Simple graphing routines for linear/polynomial regression data points and curves.
  Can be extended for the other curve types -> do this yourself.                    }
unit SGraph;

interface

uses
  WinTypes, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  Menus, ExtCtrls, Statistc, StdCtrls;

type
  TStatGraphForm = class(TForm)
    DrawPad: TPaintBox;
    MainMenu1: TMainMenu;
    File1: TMenuItem;
    Axes1: TMenuItem;
    ShowAxes1: TMenuItem;
    ColorDialog1: TColorDialog;
    Colors1: TMenuItem;
    Background1: TMenuItem;
    Axes2: TMenuItem;
    Curve1: TMenuItem;
    Pixels1: TMenuItem;
    Zoom1: TMenuItem;
    ZoomIn1: TMenuItem;
    ZoomOut1: TMenuItem;
    Label1: TLabel;
    Exit1: TMenuItem;
    procedure DrawPadPaint(Sender: TObject);
    procedure ShowAxes1Click(Sender: TObject);
    procedure FormCreate(Sender: TObject);
    procedure SetColor(Sender: TObject);
    procedure ZoomIn1Click(Sender: TObject);
    procedure ZoomOut1Click(Sender: TObject);
    procedure DrawPadMouseMove(Sender: TObject; Shift: TShiftState; X,
      Y: Integer);

    procedure FormClose(Sender: TObject; var Action: TCloseAction);
    procedure FormActivate(Sender: TObject);
    procedure Exit1Click(Sender: TObject);

  private
    { Private declarations }
    AxesColor, CurveColor, PixelColor, OldAxesColor, BackgroundColor: TColor;
    ScaleX, ScaleY: integer;
    procedure PlotAxes;
    procedure PlotPixels(X, Y: StatVector);
    procedure PlotLinRegCurve(CType: CurveFittingType; a, b: extended);
    procedure PlotPolynomCurve;
  public
    { Public declarations }
  end;

var
  StatGraphForm: TStatGraphForm;
  SX, SY: StatVector;
  CurType: CurveFittingType;
  ca, cb: extended;
  DrawLinRegCurve: boolean;
  Polyn: RealVector;

implementation

{$R *.DFM}


procedure TStatGraphForm.PlotAxes;
var x, y, i: integer;
begin
     x:= DrawPad.width;
     y:= DrawPad.height;
     with DrawPad.Canvas do
     begin
     Pen.Color := AxesColor;
     for i:= 1 to x - 1 do
     begin
     moveto(i,Round(y/2));
     lineto(i+1,Round(y/2));
     end;
     for i:= 1 to y - 1 do
     begin
     moveto(Round(x/2),i);
     lineto(Round(x/2),i+1);
     end;
     y:= x div 2;
     for i:= 1 to y do
     begin
     moveto(Round(x/2) + i*ScaleX, (DrawPad.height div 2) - 2);
     lineto(Round(x/2) + i*ScaleX, (DrawPad.height div 2) + 2);
     moveto(Round(x/2) - i*ScaleX, (DrawPad.height div 2) - 2);
     lineto(Round(x/2) - i*ScaleX, (DrawPad.height div 2) + 2);
     end;
     y:= DrawPad.height div 2;
     for i:= 1 to y do
     begin
     moveto((DrawPad.Width div 2) - 2, Round(DrawPad.height/2) + i*ScaleY);
     lineto((DrawPad.Width div 2) + 2, Round(DrawPad.height/2) + i*ScaleY);
     moveto((DrawPad.Width div 2) - 2, Round(DrawPad.height/2) - i*ScaleY);
     lineto((DrawPad.Width div 2) + 2, Round(DrawPad.height/2) - i*ScaleY);
     end;
     end;
end;

{ Plot data points. }
procedure TStatGraphForm.PlotPixels(X, Y: StatVector);
var i, dx, dy: integer;
begin
     try
     dx:= DrawPad.width;
     dy:= DrawPad.height;
     with DrawPad.Canvas do
     begin
     for i:= 1 to X.Dimension do
     Pixels[round((dx/2 + X.Vector^[i]*ScaleX)),round(dy/2 - Y.Vector^[i]*ScaleY)]:= PixelColor;
     end;
     except on EZeroDivide do abort;
     end;
end;

{ Plot linear regression curve. }
procedure TStatGraphForm.PlotLinRegCurve(CType: CurveFittingType; a, b: extended);
var i: integer;
begin
     {Linear: y = a + b*x
      Exponential: y = a*exp(b*x)
      Logarithmic: y = a + b*ln(x)
      Power: y = a*x^b (a>0).        }
  try
     with DrawPad.Canvas do
     begin
     Pen.Color := CurveColor;
     case CType of
     Linear:begin
              for i:= 1 to DrawPad.Width do
              begin
              moveto(i,Round(DrawPad.height/2 - ScaleY*(a + b*(i - DrawPad.width/2)/ScaleY)));
              lineto(i+1,Round(DrawPad.height/2 - ScaleY*(a + b*(i - DrawPad.width/2)/ScaleY)));
              end;
            end;
     Exponential:begin
              for i:= 1 to DrawPad.Width do
              begin
              moveto(i,Round(DrawPad.height/2 - ScaleY*(a*exp(b*(i - DrawPad.width/2)/ScaleY))));
              lineto(i+1,Round(DrawPad.height/2 - ScaleY*(a*exp(b*(i - DrawPad.width/2)/ScaleY))));
              end;
            end;
     Power:begin
              for i:= DrawPad.Width div 2 + 1 to DrawPad.Width do
              begin
              moveto(i,Round(DrawPad.height/2 - ScaleY*(a*exp(b*ln((i - DrawPad.width/2)/ScaleY)))));
              lineto(i+1,Round(DrawPad.height/2 - ScaleY*(a*exp(b*ln((i - DrawPad.width/2)/ScaleY)))));
              end;
            end;
     Logarithmic:begin
              for i:= DrawPad.Width div 2 + 1 to DrawPad.Width do
              begin
              moveto(i,Round(DrawPad.height/2 - ScaleY*(a + b*ln((i - DrawPad.width/2)/scaleY))));
              lineto(i+1,Round(DrawPad.height/2 - ScaleY*(a + b*ln((i - DrawPad.width/2)/scaleY))));
              end;
            end;
     end;
  end;
  except on EInvalidOp do abort;
     end;
end;

{ Plot a polynom. }
procedure TStatGraphForm.PlotPolynomCurve;
var i: integer;
    x: extended;
begin
     try
     with DrawPad.Canvas do
     begin
     Pen.Color := CurveColor;
     for i:= 1 to DrawPad.Width do
     begin
     x:= PolynomValue(Polyn, ((i - DrawPad.width/2)/ScaleY));
     moveto(i,Round(DrawPad.height/2 - x*scaley));
     lineto(i+1,Round(DrawPad.height/2 - x*scaley));
     end;
     end;
     except on EInvalidOp do
     abort;
     end;
end;

procedure TStatGraphForm.DrawPadPaint(Sender: TObject);
begin
     DrawPad.Canvas.Brush.Color:= BackgroundColor;
     StatGraphForm.Color:= BackgroundColor;
     if ShowAxes1.Checked = false then AxesColor:= BackgroundColor
     else AxesColor:= OldAxesColor;
     PlotAxes;
     if DrawLinRegCurve = true then PlotLinRegCurve(CurType, ca, cb)
     else PlotPolynomCurve;
     PlotPixels(SX, SY);
end;

procedure TStatGraphForm.ShowAxes1Click(Sender: TObject);
begin
     if ShowAxes1.Checked = true then ShowAxes1.Checked:= false
     else ShowAxes1.Checked:= true;
     DrawPadPaint(Sender);
end;

procedure TStatGraphForm.FormCreate(Sender: TObject);
begin
     AxesColor:= clGreen;
     PixelColor:= clYellow;
     CurveColor:= clBlue;
     OldAxesColor:= ClGreen;
     BackgroundColor:= clBlack;
     Label1.Font.Color:= CurveColor;
end;

procedure TStatGraphForm.SetColor(Sender: TObject);
begin
     if ColorDialog1.Execute then
     begin
     if Sender = Background1 then BackgroundColor:= ColorDialog1.Color
     else if Sender = Axes2 then
     begin
     AxesColor:= ColorDialog1.Color;
     OldAxesColor:= AxesColor;
     end
     else if Sender = Curve1 then
     begin
     CurveColor:= ColorDialog1.Color;
     Label1.Font.Color:= CurveColor;
     end
     else PixelColor:= ColorDialog1.Color;
     DrawPadPaint(Sender);
     end;
end;

procedure TStatGraphForm.ZoomIn1Click(Sender: TObject);
begin
     ScaleX:= ScaleX * 2;
     ScaleY:= ScaleY * 2;
     StatGraphForm.Caption:= 'StatGraph ' + ' ZoomFactor: ' + IntToStr(ScaleY);
     DrawPad.Refresh;
     DrawPadPaint(Sender);
end;

procedure TStatGraphForm.ZoomOut1Click(Sender: TObject);
var OldX, OldY: integer;
begin
     OldX:= ScaleX;
     OldY:= ScaleY;
     if OldX > 1 then
     begin
     ScaleX:= ScaleX div 2;
     ScaleY:= ScaleY div 2;
     StatGraphForm.Caption:= 'StatGraph ' + ' ZoomFactor: ' + IntToStr(ScaleY);
     DrawPad.Refresh;
     DrawPadPaint(Sender);
     end
     else
     ShowMessage('Can not zoom more out!');
end;

procedure TStatGraphForm.DrawPadMouseMove(Sender: TObject; Shift: TShiftState; X,
  Y: Integer);
var dx,dy: extended;
    s: string;
begin
     dx:= X - DrawPad.Width/2;
     dy:= DrawPad.Height/2 - Y;
     s:= 'X: ' + FloatToStrF(dx/ScaleX, ffGeneral, 3, 3) +
                    ' Y: ' + FloatToStrF(dy/ScaleY, ffGeneral, 3, 3);
     Label1.Caption:= s;
end;

procedure TStatGraphForm.FormClose(Sender: TObject;
  var Action: TCloseAction);
begin
     DestroyStatVector(SX);
     DestroyStatVector(SY);
     showmessage('SX and SY are out for lunch.');
     { now, SX and SY are gone! }
end;

{ Scale the coordinates so the curve/data points fit in the window. }
procedure TStatGraphForm.FormActivate(Sender: TObject);
var ma, mi: extended;
begin
     mi:= MinValue(Sy);
     ma:= MaxValue(Sy);
     ma:= ma - mi;
     ScaleX:= Round(DrawPad.Width/(ma*10));
     ScaleY:= Round(DrawPad.Width/(ma*10));
     if ScaleX = 0 then ScaleX:= 10; { May not be 0! }
     if ScaleY = 0 then ScaleY:= 10;
     StatGraphForm.Caption:= 'StatGraph ' + ' ZoomFactor: ' + IntToStr(ScaleY);
end;

procedure TStatGraphForm.Exit1Click(Sender: TObject);
begin
     Close;
end;

end.
